public class scalec {
  int scale[];
  scalec(int n) {
    scale = new int[n];
  }
  void init() {
    int octave_cents = main_app.octave_cents;
    int et = main_app.notes_per_octave;
    for (int i = 0;i < et;i++) {
      scale[i] = (int) ((((double)octave_cents)*i)/et);
    }
  }
  int get(int i) {
    return scale[i];
  }
  void set(int i,int v) {
    scale[i] = v;
  }
}

